#!/usr/bin/perl -w
# Pick up a "Euteleostomi" Node including Human_.

# perl NTFdaughterPicker.pl CAMK2G_H.0

use strict;
use warnings;

my $protID      = "ENSP00000410298";
my $nodeName    = "Euteleostomi";
#my $nodeName   = "Bilateria";
my $treeFileNHX = "geneTreeNHX.txt";
my $ancestralDepth = 1;

##### Main program
open(IN,"$treeFileNHX");
my @infileCont = <IN>;
close IN;
my $treeNHX = shift@infileCont;

my $refAllNodes = &nodesCollectFromNHX($treeNHX);

my $orthoNode                          = &orthoNodeIdentify($nodeName, $protID, $refAllNodes);
my($daughterNode1st, $daughterNode2nd) = &daughterNodesIdentify($orthoNode,     $refAllNodes);

my $ref_ancestralNodesDecrement = &ancestralNodesCollect($orthoNode,   $refAllNodes);
my $parentNode                  = &parentNodeIdentify($ancestralDepth, $ref_ancestralNodesDecrement);


## Print out
print "\n";
print "orthoNode: $nodeName, $protID\n";
print "  $orthoNode->[0] spp.\n";
print "  $orthoNode->[2]\n";
#print "  @{$orthoNode->[1]}\n";
print "\n";

print "DaughterNode1:\n";
print "  $daughterNode1st->[0] spp.\n";
print "  $daughterNode1st->[2]\n";
#print "  @{$daughterNode1st->[1]}\n";
print "\n";

print "DaughterNode2:\n";
print "  $daughterNode2nd->[0] spp.\n";
print "  $daughterNode2nd->[2]\n";
#print "  @{$daughterNode2nd->[1]}\n";
print "\n";

print "ParentNode:\n";
print "  $parentNode->[0] spp.\n";
print "  $parentNode->[2]\n";
#print "  @{$parentNode->[1]}\n";
print "\n";

exit;


sub nodesCollectFromNHX {
  my $treeNHX = shift@_;

  my $treeSR = $treeNHX;
  my @nodes;
  while ($treeSR =~ s/\(([^\(\)]+)\)(.*?\[.*?\])/$1/) {   
    chomp($treeSR);
    my $leaves = $1;        
    my $exp    = $2;         
    $leaves =~ s/\[.*?\]//g;      # Delete exp [...]
    $leaves =~ s/:\d+\.\d+//g;    # Delete blanch lengths
    $exp    =~ s/:\d+\.\d+//g;     
    $leaves =~ s/E-\d*//g;        # Delete blanch lengths with E-
    my @eachLeaves = split(/,/, $leaves);
    push (@nodes,[scalar(@eachLeaves), \@eachLeaves, $exp]);
  }
  my @sortedNodes = sort{$b->[0] <=> $a->[0]}@nodes;
  
  ## Add leaves as nodes
  my $largestNode = $sortedNodes[0];
  my @largestNodeLeaves = @{$largestNode->[1]};
  foreach my $leaf (@largestNodeLeaves){
    my @tempLeafNode = ($leaf);
    push(@sortedNodes, [1, \@tempLeafNode, 1]);
  }

  return (\@sortedNodes);
}


sub orthoNodeIdentify {
  my ($nodeName, $protIDSR, $ref_allNodesSR) = @_;
  my @allNodesSR = @$ref_allNodesSR;
  my @keyNodes   = ();
  foreach my $node (@allNodesSR) {
    if($node->[2] =~ /S=$nodeName:/){
      if (grep/_$protIDSR$/,@{$node->[1]}){
        push(@keyNodes, $node);
      }
    }
  }
  unless (@keyNodes){
    print "Stopped. No orthoNode was found for $nodeName and $protIDSR.\n";
    exit;  
  } 
  else {
    my $orthoNodeSR = pop@keyNodes;
    return ($orthoNodeSR);
  }
}


sub isIncluded {
    my ($ref_checkNodeLeave, $ref_focalNodeLeave) = @_;
    my @checkNodeLeave = @$ref_checkNodeLeave;
    my @focalNodeLeave = @$ref_focalNodeLeave;
    foreach my $ckeckNodeLeaf (@checkNodeLeave) {
      unless(grep m|\Q$ckeckNodeLeaf\E|, @focalNodeLeave){
        return 0;
      }
    }
    return 1;
}


sub isIndependent {
    my ($ref_checkNodeLeave, $ref_focalNodeLeave) = @_;
    my @checkNodeLeave = @$ref_checkNodeLeave;
    my @focalNodeLeave = @$ref_focalNodeLeave;

    foreach my $ckeckNodeLeaf (@checkNodeLeave) {
      if(grep m|\Q$ckeckNodeLeaf\E|, @focalNodeLeave){
        return 0;
      }
    }
    return 1;
}


sub sisterNodeIdentify{
  my ($ref_childNodesSR, $daughterNode1st) = @_;
  my @childNodesSR =  @$ref_childNodesSR;
  my $daughterNode2nd = "";
  foreach my $eachChildNode (@childNodesSR) {
    if (&isIndependent($eachChildNode->[1], $daughterNode1st->[1])){
      $daughterNode2nd = $eachChildNode;
      last;
    }
  }
  return($daughterNode2nd);
}


sub childNodesCollect{
  my ($ref_allNodesSR, $orthoCladSR) = @_;
  my @allNodesSR   =  @$ref_allNodesSR;
  my @childNodesSR = ();
  foreach my $eachNode (@allNodesSR) {
    if (&isIncluded($eachNode->[1], $orthoCladSR->[1])){
      push(@childNodesSR, $eachNode);
    }
  }
  return(\@childNodesSR);
}


sub daughterNodesIdentify{
  my ($orthoNodeSR, $refAllNodesSR) = @_;
  my @allNodesSR = @$refAllNodesSR;

  my $rec_childNodes = &childNodesCollect(\@allNodesSR, $orthoNodeSR);
  my @childNodes = @$rec_childNodes;

  my $daughterNode1st = $childNodes[1];
  my $daughterNode2nd = &sisterNodeIdentify(\@childNodes, $daughterNode1st);
  return($daughterNode1st, $daughterNode2nd);
}


sub ancestralNodesCollect{
  my ($orthoNodeSR, $ref_allNodes) = @_;
  my @allNodes = @$ref_allNodes;
  my @ancestralNodesDecrementSR = ();
  foreach my $eachNode (@allNodes) {
    if (&isIncluded($orthoNodeSR->[1], $eachNode->[1])){
      push(@ancestralNodesDecrementSR, $eachNode);
    }
  }
  return (\@ancestralNodesDecrementSR);
}


sub parentNodeIdentify{
  my ($ancestralDepthSR, $ref_ancestralNodesDecrementSR) = @_;
  my @ancestralNodesSRIncrement = sort{$a->[0] <=> $b->[0]}@$ref_ancestralNodesDecrementSR;
  my $parentalNodeSR = "";

  for (my $depth = $ancestralDepthSR; $depth >= 0; $depth--){
    if ($ancestralNodesSRIncrement[$depth]){
      $parentalNodeSR = $ancestralNodesSRIncrement[$depth];
      last;
    }
  }
  return ($parentalNodeSR);
}
